// -*- C++ -*-
/*!
 *
 * @file BufferStatus.h
 * @brief Buffer status enum definition
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 */

#ifndef RTC_BUFFERSTATUS_H
#define RTC_BUFFERSTATUS_H

namespace RTC
{
  /*!
   * @if jp
   * @class BufferStatus mixin class
   * @brief BufferStatus mixin NX
   *
   * ̃NX́Aenum`ꂽ^[R[hAobt@֘ATuN
   * Xŋʗp邽߂ mixin NXłB̃^[R[hg
   * pNXł́ABufferStatus NXpublic pA define
   * Ă BUFFERSTATUS_ENUM NXɋLq邱Ƃŗp\Ƃ
   * BɂAenum  ReturnCode_t ^Ƃ typedef AȌ
   * ReturnCode_t 𗘗pł悤ɂƂƂɁAOԂ enum `
   * ꂽeʎq𓖊YNXOԓɓB
   *
   * @else
   * @class BufferStatus mixin class
   * @brief BufferStatus mixin class
   *
   * This is a mixin class to provide enumed return codes that are
   * commonly utilised in buffer realted sub-classes. To use this
   * class, sub-class should inherit this class as a public super
   * class, and declare BUFFERSTATUS_ENUM defined below. Consequently,
   * ReturnCode_t type that is typedefed by this macro can be used in
   * the sub-class, and enumed identifiers are imported to the class's
   * namespace.
   *
   * @endif
   */
  class BufferStatus
  {
  public:
    /*!
     * @if jp
     * @brief BufferStatus ^[R[h
     *
     * obt@֘ÃNXŋʂ̃^[R[h
     *
     * - BUFFER_OK:            I
     * - BUFFER_ERROR:         obt@G[
     * - BUFFER_FULL:          obt@t
     * - BUFFER_EMPTY:         obt@GveB
     * - NOT_SUPPORTED:        T|[g@\
     * - TIMEOUT:              ^CAEg
     * - PRECONDITION_NOT_MET: O𖞂ĂȂ
     *
     * @else
     * @brief DataPortStatus return codes
     *
     * Common return codes for buffer classes.
     *
     * - BUFFER_OK:            Normal return
     * - BUFFER_ERROR:         Buffer error
     * - BUFFER_FULL:          Buffer full
     * - BUFFER_EMPTY:         Buffer empty
     * - NOT_SUPPORTED:        Not supported function
     * - TIMEOUT:              Timeout
     * - PRECONDITION_NOT_MET: Precodition not met
     *
     * @endif
     */
    enum Enum
      {
        BUFFER_OK = 0,
        BUFFER_ERROR,
        BUFFER_FULL,
        BUFFER_EMPTY,
        NOT_SUPPORTED,
        TIMEOUT,
        PRECONDITION_NOT_MET
      };

    /*!
     * @if jp
     *
     * @brief BufferStatus ^[R[h𕶎ɕϊ
     *
     * BufferStatus ^[R[h𕶎ɕϊ
     *
     * @param status ϊΏ BufferStatus ^[R[h
     *
     * @return ϊ
     *
     * @else
     *
     * @brief Convert BufferStatus into the string.
     *
     * Convert BufferStatus into the string.
     *
     * @param status The target BufferStatus for transformation
     *
     * @return Trnasformation result of string representation
     *
     * @endif
     */
    static const char* toString(Enum status)
    {
      const char* str[] = {
        "BUFFER_OK",
        "BUFFER_ERROR",
        "BUFFER_FULL",
        "BUFFER_EMPTY",
        "NOT_SUPPORTED",
        "TIMEOUT",
        "PRECONDITION_NOT_MET"
      };
      return str[status];
    }
  };
}; // namespace RTC

#define BUFFERSTATUS_ENUM \
  typedef ::RTC::BufferStatus::Enum ReturnCode;       \
  using ::RTC::BufferStatus::BUFFER_OK;               \
  using ::RTC::BufferStatus::BUFFER_ERROR;            \
  using ::RTC::BufferStatus::BUFFER_FULL;             \
  using ::RTC::BufferStatus::BUFFER_EMPTY;            \
  using ::RTC::BufferStatus::NOT_SUPPORTED;           \
  using ::RTC::BufferStatus::TIMEOUT;                 \
  using ::RTC::BufferStatus::PRECONDITION_NOT_MET;

#endif // RTC_BUFFERSTATUS_H
