// -*- C++ -*-
/*!
 * @file DataPortStatus.h
 * @brief DataPortStatus class
 * @date $Date$
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTC_DATAPORTSTATUS_H
#define RTC_DATAPORTSTATUS_H

#include <vector>

namespace RTC
{
  /*!
   * @if jp
   * @class DataPortStatus mixin class
   * @brief DataPortStatus mixin NX
   *
   * ̃NX́Aenum`ꂽ^[R[hAf[^|[g֘ÃT
   * uNXŋʗp邽߂ mixin NXłB̃^[R[
   * hgpNXł́ADataPortStatus NXpublic pA
   * define ĂDATAPORTSTATUS_ENUM NXɋLq邱Ƃŗp
   * \ƂȂBɂAenum  ReturnCode_t ^Ƃ typedef A
   * ȌReturnCode_t 𗘗pł悤ɂƂƂɁAOԂ enum
   * `ꂽeʎq𓖊YNXOԓɓB
   *
   * @else
   * @class DataPortStatus mixin class
   * @brief DataPortStatus mixin class
   *
   * This is a mixin class to provide enumed return codes that are
   * commonly utilised in data port related sub-classes. To use this
   * class, sub-class should inherit this class as a public super
   * class, and declare DATAPORTSTATUS_ENUM defined
   * below. Consequently, ReturnCode_t type that is typedefed by this
   * macro can be used in the sub-class, and enumed identifiers are
   * imported to the class's namespace.
   *
   * @endif
   */
  class DataPortStatus
  {
  public:
    /*!
     * @if jp
     * brief DataPortStatus ^[R[h
     *
     * f[^|[g֘ÃNXŋʂ̃^[R[h
     *  
     * - PORT_OK:              I
     * - PORT_ERROR:           ُI
     * - BUFFER_ERROR:         obt@G[
     * - BUFFER_FULL:          obt@t
     * - BUFFER_EMPTY:         obt@GveB
     * - BUFFER_TIMEOUT:       obt@^CAEg
     * - SEND_FULL:            f[^𑗂葤obt@t
     * - SEND_TIMEOUT:         f[^𑗂葤^CAEg
     * - RECV_EMPTY:           f[^M悤Ƃf[^
     * - RECV_TIMEOUT:         f[^M悤Ƃ^CƂ
     * - INVALID_ARGS:         sȈ
     * - PRECONDITION_NOT_MET: O𖞂ĂȂ
     * - CONNECTION_LOST:      ڑؒfꂽ
     * - UNKNOWN_ERROR:        sȃG[
     *
     * f[^|[g̃f[^oH̃G[ĂяoփG[
     * `邽߂ɂ̃G[R[hgpBɁA`H̃G
     * [A`̃G[Ȃǂl邪Ae̊EʂŔG
     * [ȉɗ񋓂B
     *
     * (1) Push^
     *  a) InPortConsumer  Publisher/Activity ԂŔ郊^[R[h
     *     PORT_OK, PORT_ERROR, SEND_FULL, SEND_TIMEOUT, CONNECTION_LOST,
     *     UNKNOWN_ERROR
     *
     *  b) Activity  OutPort  Buffer/Connector ԂŔ郊^[R[h
     *     PORT_OK, PORT_ERROR, BUFFER_ERROR, BUFFER_FULL, BUFFER_TIMEOUT,
     *     UNKNOWN_ERROR, 
     *
     * (2) Pull^
     *  a) Activity  InPort ̊ԂŔ郊^[R[h
     *     PORT_OK, PORT_ERROR, RECV_EMPTY, RECV_TIMEOUT, CONNETION_LOST,
     *     UNKNOWN_ERROR
     *
     * e֐Ԃ^[R[h͊֐Ƃ̃t@XQƂ̂ƁB
     *
     * @else
     * @brief DataPortStatus return codes
     *
     * Common return codes for data ports related classes.
     *
     * - PORT_OK:              Normal return
     * - PORT_ERROR:           Error return
     * - BUFFER_ERROR:         Buffer error
     * - BUFFER_FULL:          Buffer full
     * - BUFFER_EMPTY:         Buffer empty
     * - BUFFER_TIMEOUT:       Buffer timeout
     * - SEND_FULL:            Buffer full although OutPort tried to send data
     * - SEND_TIMEOUT:         Timeout although OutPort tried to send data
     * - RECV_EMPTY:           Buffer empty although InPort tried to receive
     *                         data
     * - RECV_TIMEOUT:         Timeout although InPort tried to receive data
     * - INVALID_ARGS:         Invalid arguments
     * - PRECONDITION_NOT_MET: Precondition not met
     * - CONNECTION_LOST:      Connection has been lost
     * - UNKNOWN_ERROR:        Unknown error
     *
     * This error codes might be used to propagate error status from
     * the error occurring point to the function caller in the data
     * stream path. It would occur in data-transfer path and data
     * receiver/sender. The errors that occur in the interface of each
     * portion of data port are shown below.
     *
     * (1) Push Type
     *  a) The return codes between InPortConsumer and Publisher/Activity
     *     PORT_OK, PORT_ERROR, SEND_FULL, SEND_TIMEOUT, CONNECTION_LOST,
     *     UNKNOWN_ERROR
     *  b) The return codes between Activity and Buffer/Connector of OutPort
     *     PORT_OK, PORT_ERROR, BUFFER_ERROR, BUFFER_FULL, BUFFER_TIMEOUT,
     *     UNKNOWN_ERROR, 
     *
     * (2) Pull Type
     *  a) The return codes between Activity and InPort
     *     PORT_OK, PORT_ERROR, RECV_EMPTY, RECV_TIMEOUT, CONNETION_LOST,
     *     UNKNOWN_ERROR
     *
     * See function references for detailed return codes for each function.
     *
     * @endif
     */
    enum Enum
      {
        PORT_OK = 0,
        PORT_ERROR,
        BUFFER_ERROR,
        BUFFER_FULL,
        BUFFER_EMPTY,
        BUFFER_TIMEOUT,
        SEND_FULL,
        SEND_TIMEOUT,
        RECV_EMPTY,
        RECV_TIMEOUT,
        INVALID_ARGS,
        PRECONDITION_NOT_MET,
        CONNECTION_LOST,
        UNKNOWN_ERROR
      };

    /*!
     * @if jp
     *
     * @brief DataPortStatus ^[R[h𕶎ɕϊ
     *
     * DataPortStatus ^[R[h𕶎ɕϊ
     *
     * @param status ϊΏ DataPortStatus ^[R[h
     *
     * @return ϊ
     *
     * @else
     *
     * @brief Convert DataPortStatus into the string.
     *
     * Convert DataPortStatus into the string.
     *
     * @param status The target DataPortStatus for transformation
     *
     * @return Trnasformation result of string representation
     *
     * @endif
     */
    static const char* toString(DataPortStatus::Enum status)
    {
      const char* str[] = {
        "PORT_OK",
        "PORT_ERROR",
        "BUFFER_ERROR",
        "BUFFER_FULL",
        "BUFFER_EMPTY",
        "BUFFER_TIMEOUT",
        "SEND_FULL",
        "SEND_TIMEOUT",
        "RECV_EMPTY",
        "RECV_TIMEOUT",
        "INVALID_ARGS",
        "PRECONDITION_NOT_MET",
        "CONNECTION_LOST",
        "UNKNOWN_ERROR"
      };
      return str[status];
    }
  };
  
  typedef std::vector<DataPortStatus::Enum> DataPortStatusList;

};

#define DATAPORTSTATUS_ENUM \
  typedef ::RTC::DataPortStatus::Enum ReturnCode;       \
  using ::RTC::DataPortStatus::PORT_OK;                 \
  using ::RTC::DataPortStatus::PORT_ERROR;              \
  using ::RTC::DataPortStatus::BUFFER_FULL;             \
  using ::RTC::DataPortStatus::BUFFER_EMPTY;            \
  using ::RTC::DataPortStatus::BUFFER_TIMEOUT;          \
  using ::RTC::DataPortStatus::SEND_FULL;               \
  using ::RTC::DataPortStatus::SEND_TIMEOUT;            \
  using ::RTC::DataPortStatus::RECV_EMPTY;              \
  using ::RTC::DataPortStatus::RECV_TIMEOUT;            \
  using ::RTC::DataPortStatus::INVALID_ARGS;            \
  using ::RTC::DataPortStatus::PRECONDITION_NOT_MET;    \
  using ::RTC::DataPortStatus::CONNECTION_LOST;         \
  using ::RTC::DataPortStatus::UNKNOWN_ERROR;

#endif // RTC_DATAPORTSTATUS_H
