// -*- C++ -*-
/*!
 * @file NVUtil.h
 * @brief NameValue and NVList utility functions
 * @date $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: NVUtil.h 1808 2010-01-27 07:39:54Z fsi-katami $
 *
 */

#ifndef NVUTIL_NVUTIL_H
#define NVUTIL_NVUTIL_H

#include <string>
#include <iostream>
#include <coil/Properties.h>
#include <rtm/idl/SDOPackageSkel.h>

/*!
 * @if jp
 * @namespace NVUtil
 *
 * @brief NameValue p[eBeB
 *
 * NameValue ɑ΂Ẵ[eBeB֐񋟂B
 *
 * @else
 *
 * @namespace NVUtil
 *
 * @brief Utility for NameValue
 *
 * This class provides the utility function of NameValue. 
 *
 * @endif
 */
namespace NVUtil
{
  /*!
   * @if jp
   *
   * @brief NameValue 𐶐
   *
   * ̃Iy[VNameValue쐬B
   * CORBA::Char, CORBA::Boolean, CORBA::Octet ͍쐬łȂB
   * ̒l newNVChar(), newNVBool(), newNVOctet() ō쐬B
   *
   * @param name NameValue  name
   * @param value NameValue  value
   *
   * @return NameValue
   *
   * @else
   *
   * @brief Create NameValue
   *
   * This operation creates NameValue.
   * CORBA::Char, CORBA::Boolean, CORBA::Octet creation is not supported.
   * These type of NameValue should be created by using 
   * newNVChar(), newNVBool(), newNVOctet() functions.
   *
   * @param name Name of NameValue
   * @param value The value of NameValue
   *
   * @return NameValue
   *
   * @endif
   */
  template <class Value>
  SDOPackage::NameValue newNV(const char* name, Value value)
  {
    SDOPackage::NameValue nv;
    nv.name = CORBA::string_dup(name);
    nv.value <<= value;
    return nv;
  }
  
  /***
   * @if jp
   *
   * @brief value  CORBA::string  NameValue 𐶐
   *
   * ̃Iy[Vf value  CORBA::string  NameValue쐬B
   *
   * @param name NameValue  name
   * @param value NameValue  value
   *
   * @return NameValue
   *
   * @else
   *
   * @brief Create NameValue typed CORBA::string
   *
   * This operation creates NameValue typed CORBA::string.
   *
   * @param name Name of NameValue
   * @param value The value of NameValue
   *
   * @return NameValue
   *
   * @endif
   */
  /*
    template <>
    SDOPackage::NameValue newNV(const char* name, const char* value)
    {
    SDOPackage::NameValue nv;
    nv.name = CORBA::string_dup(name);
    nv.value <<= value;
    return nv;
    }
  */ 
  
  /*!
   * @if jp
   *
   * @brief value  CORBA::Char  NameValue 𐶐
   *
   * ̃Iy[Vf value  CORBA::Char  NameValue쐬B
   *
   * @param name NameValue  name
   * @param value NameValue  value
   *
   * @return NameValue
   *
   * @else
   *
   * @brief Create NameValue typed CORBA::Char
   *
   * This operation creates NameValue typed CORBA::Char.
   *
   * @param name Name of NameValue
   * @param value The value of NameValue
   *
   * @return NameValue
   *
   * @endif
   */
  SDOPackage::NameValue newNVChar(const char* name, const CORBA::Char value);
  
  /*!
   * @if jp
   *
   * @brief value  CORBA::Boolean  NameValue 𐶐
   *
   * ̃Iy[Vf value  CORBA::Boolean  NameValue쐬B
   *
   * @param name NameValue  name
   * @param value NameValue  value
   *
   * @return NameValue
   *
   * @else
   *
   * @brief Create NameValue typed CORBA::Boolean
   *
   * This operation creates NameValue typed CORBA::Boolean.
   *
   * @param name Name of NameValue
   * @param value The value of NameValue
   *
   * @return NameValue
   *
   * @endif
   */
  SDOPackage::NameValue newNVBool(const char* name,
				  const CORBA::Boolean value);
  
  /*!
   * @if jp
   *
   * @brief value  CORBA::Octet  NameValue 𐶐
   *
   * ̃Iy[V value  CORBA::Octet  NameValue쐬B
   *
   * @param name NameValue  name
   * @param value NameValue  value
   *
   * @return NameValue
   *
   * @else
   *
   * @brief Create NameValue typed CORBA::Octet
   *
   * This operation creates NameValue typed CORBA::Octet.
   *
   * @param name Name of NameValue
   * @param value The value of NameValue
   *
   * @return NameValue
   *
   * @endif
   */
  SDOPackage::NameValue newNVOctet(const char* name, const CORBA::Octet value);
  
  /*!
   * @if jp
   *
   * @brief value  CORBA::Any  NameValue 𐶐
   *
   * ̃Iy[Vf value  CORBA::Any  NameValue쐬B
   *
   * @param name NameValue  name
   * @param value NameValue  value
   *
   * @return NameValue
   *
   * @else
   *
   * @brief Create NameValue typed CORBA::Any
   *
   * This operation creates NameValue typed CORBA::Any.
   *
   * @param name Name of NameValue
   * @param value The value of NameValue
   *
   * @return NameValue
   *
   * @endif
   */
  SDOPackage::NameValue newNVAny(const char* name, const CORBA::Any& value);
  
  /*!
   * @if jp
   *
   * @brief Properties  NVList փRs[
   *
   * ̃Iy[V Properties  NVList փRs[B
   * NVList  value ͑S CORBA::string ^ƂăRs[B
   *
   * @param nv Properties ̒li[ NVList
   * @param prop Rs[ Properties
   *
   * @else
   *
   * @brief Copy the properties to NVList
   *
   * This operation copies the properties to NVList.
   * All NVList's values are copied as CORBA::string.
   *
   * @param nv NVList to store properties values
   * @param prop Properties that is copies from
   *
   * @endif
   */
  void copyFromProperties(SDOPackage::NVList& nv, const coil::Properties& prop);
  
  /*!
   * @if jp
   *
   * @brief NVList  Properties փRs[
   *
   * ̃Iy[V NVList  Properties փRs[B
   *
   * @param prop NVList ̒li[ Properties
   * @param nv Rs[ NVList
   *
   * @else
   *
   * @brief Copy NVList to the Proeprties
   *
   * This operation copies NVList to properties.
   *
   * @param prop Properties to store NVList values
   * @param nv NVList of copy source
   *
   * @endif
   */
  void copyToProperties(coil::Properties& prop, const SDOPackage::NVList& nv);
  
  /*!
   * @if jp
   *
   * @brief NVList  Properties ֕ϊ
   *
   * ̃Iy[V NVList  Properties ֕ϊB
   *
   * @param nv ϊ NVList
   *
   * @return ϊProperty
   *
   * @else
   *
   * @brief Transform NVList to the properties
   *
   * This operation transforms NVList to properties
   *
   * @param nv NVList of tranformation source
   *
   * @return Transformation result Property
   *
   * @endif
   */
  coil::Properties toProperties(const SDOPackage::NVList& nv);
  
  /*!
   * @if jp
   *
   * @brief NVList  name Ŏw肳ꂽ value Ԃ
   *
   * ̃Iy[V name Ŏw肳ꂽ value  Any ^ŕԂB
   * w肵̗̂vf݂Ȃꍇ͗O𔭐B
   *
   * @param nv Ώۂ NVList
   * @param name 閼O
   *
   * @return 
   *
   * @else
   *
   * @brief Return the value specified by name from NVList
   *
   * This operation returns Any type of value specified by name.
   * When an element of specified name doesn't exist, the exception will occur.
   *
   * @param nv The target NVList for the find
   * @param name  Name for the find
   *
   * @return Find result
   *
   * @endif
   */
  const CORBA::Any& find(const SDOPackage::NVList& nv, const char* name);
  
  /*!
   * @if jp
   *
   * @brief name Ŏw肳ꂽvf̃CfbNXԂ
   *
   * ̃Iy[V name Ŏw肳ꂽvfi[Ăʒu
   * CfbNXԂB
   *
   * @param nv Ώۂ NVList
   * @param name 閼O
   *
   * @return Ώۂ̃CfbNX
   *
   * @else
   *
   * @brief Return the index of element specified by name from NVList
   *
   * This operation returns the index at the position where the element
   * specified by name is stored.
   *
   * @param nv The target NVList for the find
   * @param name  Name for the find
   *
   * @return Index of target object for the find
   *
   * @endif
   */
  const CORBA::Long find_index(const SDOPackage::NVList& nv, const char* name);
  
  /*!
   * @if jp
   *
   * @brief w肳ꂽ name  value ̌^ string ł邩؂
   *
   * ̃Iy[V name Ŏw肳ꂽ value ̌^ CORBA::string
   * ǂ bool lŕԂB
   *
   * @param nv Ώۂ NVList
   * @param name 閼O
   *
   * @return string،(string:trueAȊO:false)
   *
   * @else
   *
   * @brief Validate whether value type specified by name is string type
   *
   * This operation returns the bool value by checking whether the type of
   * value specified with name is CORBA::string.
   *
   * @param nv The target NVList for the search
   * @param name Name for the search
   *
   * @return String validation result (String:true, Else:false)
   *
   * @endif
   */
  bool isString(const SDOPackage::NVList& nv, const char* name);
  
  /*!
   * @if jp
   *
   * @brief w肳ꂽ name  value ̒lw肵ƈv邩؂
   *
   * ̃Iy[V name Ŏw肳ꂽ value ̌^ CORBA::string
   * ǂ𔻒fA  CORBA::string łꍇɂ͎w肵ƈv邩
   * bool lŕԂB
   *
   * @param nv Ώۂ NVList
   * @param name 閼O
   * @param value rΏە
   *
   * @return ،(ƈv:trueAv:false)
   *
   * @else
   *
   * @brief Check whether the value of specified name matches the specified
   *        string
   *
   * This operation checks whether the value specified with name is 
   * CORBA::string and returns the bool value which matches spcified string.
   *
   * @param nv The target NVList for the search
   * @param name Name for the search
   * @param value String value to compare
   *
   * @return Check result (Match:true, Unmatch:false)
   *
   * @endif
   */
  bool isStringValue(const SDOPackage::NVList& nv, const char* name,
		     const char* value);
  
  /*!
   * @if jp
   *
   * @brief w肳ꂽ name  NVList  string ƂĕԂB
   *
   * ̃Iy[V name Ŏw肳ꂽ NVList ̒l string ŕԂB
   * Aname Ŏw肵 value ̒l CORBA::string łȂ΁A
   * ̕stringԂB
   *
   * @param nv Ώۂ NVList
   * @param name 閼O
   *
   * @return name ɑΉlstring^̒l
   *
   * @else
   *
   * @brief Get NVList of specifid name as string
   *
   * This operation returns string value in NVList specified by name.
   * If the value in NVList specified by name is not CORBA::string type
   * this operation returns empty string value.
   *
   * @param nv The target NVList for the search
   * @param name Name for the search
   *
   * @return String value corresponding to name
   *
   * @endif
   */
  std::string toString(const SDOPackage::NVList& nv, const char* name);
  
  /*!
   * @if jp
   *
   * @brief w肳ꂽ NVList ̗vfɒǉB
   *
   * ̃Iy[V name Ŏw肳ꂽvf value Ŏw肳ꂽ
   * ǉB
   * name Ŏw肵vfɊ value ̒lݒ肳Ăꍇɂ͉ȂB
   * name Ŏw肵vf value ̒lݒ肳ĂȂꍇ́A ","؂
   * value ̒lǉB
   * w肳ꂽlݒ肷B
   * name Ŏw肵vf݂Ȃꍇ́A NVList ̍ŌɐVȗvfǉA
   * w肳ꂽlݒ肷B
   *
   * @param nv Ώۂ NVList
   * @param name ǉΏۗvf
   * @param value ǉ镶
   *
   * @return ǉ쌋
   *
   * @else
   *
   * @brief Append the specified string to element of NVList
   *
   * This operation appends the string value specified by value to the element
   * specified by name.
   * Operate nothing when the 'value' value has already been set to the
   * element specified by name.
   * Add the 'value' value each separating by a comma "," when the 'value'
   * value is not set to the element specified by name.
   * Set the specified value.
   * Add a new element at the end of NVList, and set the specified value,
   * when the element specified by name does not exist.
   *
   * @param nv The target NVList for the search
   * @param name The target element name for the appending
   * @param value String to append
   *
   * @return Append operation result
   *
   * @endif
   */
  bool appendStringValue(SDOPackage::NVList& nv, const char* name,
			 const char* value);
  
  /*!
   * @if jp
   *
   * @brief NVList ɗvfǉB
   *
   * ̃Iy[V dest Ŏw肳ꂽ NVList  src Ŏw肳ꂽvf
   * ǉB
   *
   * @param dest ǉ NVList
   * @param src ǉ NVList
   *
   * @else
   *
   * @brief Append an element to NVList
   *
   * This operation appends elements specified by src to NVList specified
   * by dest.
   *
   * @param dest NVList to be appended
   * @param src NVList to append
   *
   * @endif
   */
  void append(SDOPackage::NVList& dest, const SDOPackage::NVList& src);
  
  /*!
   * @if jp
   *
   * @brief NVList ɐݒ肳Ăe𕶎Ƃďo͂B
   *
   * w肳ꂽ NVList ɐݒ肳ꂽe𕶎Ƃďo͂B
   * ȂAݒ肳Ăvf^ȊȌꍇɂ́A̎|(ł͂Ȃ)
   * o͂B
   *
   * @param nv o͑Ώ NVList
   *
   * @else
   *
   * @brief Print information configured in NVList as string type
   *
   * Print configured information as string type in specified NVList.
   * Also, print the reason (this is not string type) if the configured
   * element is other than string type.
   *
   * @param nv The target NVList for the print
   *
   * @endif
   */
  std::ostream& dump(std::ostream& out, const SDOPackage::NVList& nv);

  /*!
   * @if jp
   * @brief NVList ɐݒ肳Ăe𕶎ƂĕWo͂B
   * @param nv o͑Ώ NVList
   * @else
   * @brief Print information configured in NVList as a string type 
   *        to Standard Outport.
   * @param nv The target NVList for the print
   * @endif
   */
  void dump(const SDOPackage::NVList& nv);

  /*!
   * @if jp
   * @brief NVList ɐݒ肳Ăe𕶎ɂ
   * @param nv o͑Ώ NVList
   * @else
   * @brief Get information configured in NVList as a string type.
   * @param nv The target NVList for the print
   * @endif
   */
  std::string toString(const SDOPackage::NVList& nv);
  

};
#endif // NVUTIL_NVUTIL_H
