// -*- C++ -*-
/*!
 * @file  OutPortProvider.h
 * @brief OutPortProvider class
 * @date  $Date: 2007-12-31 03:08:05 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: OutPortProvider.h 1756 2010-01-22 08:43:33Z fsi-katami $
 *
 */

#ifndef RTC_OUTPORTPROVIDER_H
#define RTC_OUTPORTPROVIDER_H

#include <string>

#include <coil/Factory.h>
#include <rtm/BufferBase.h>
#include <rtm/NVUtil.h>
#include <rtm/SystemLogger.h>
#include <rtm/DataPortStatus.h>
#include <rtm/OutPortConnector.h>

namespace RTC
{
  class ConnectorListeners;
  /*!
   * @if jp
   *
   * @class OutPortProvider
   * @brief OutPortProvider
   *
   * - Port ɑ΂ĉ񋟂Ă邩錾B
   *   PortProfile  properties  Provider ɊւǉB
   *
   * () OutPort  Provide ꍇ
   *
   * OutPortCorbaProvider ȉ錾
   *  - dataport.interface_type = CORBA_Any
   *  - dataport.dataflow_type = Push, Pull
   *  - dataport.subscription_type = Once, New, Periodic
   * 
   * OutPortRawTCPProvider ȉ錾
   *  - dataport.interface_type = RawTCP
   *  - dataport.dataflow_type = Push, Pull
   *  - dataport.subscription_type = Once, New, Periodic
   * 
   * ŏII PortProfile::properties 
   *  - dataport.interface_type = CORBA_Any, RawTCP
   *  - dataport.dataflow_type = Push, Pull
   *  - dataport.subscription_type = Once, New, Periodic
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class OutPortProvider
   * @brief OutPortProvider
   *
   * - Declare what is provided to the port.
   *   Add information associated with Provider to the properties of PortProfile.
   *
   * (Example) When OutPort is provided:
   *
   * OutPortCorbaProvider declares the following:
   *  - dataport.interface_type = CORBA_Any
   *  - dataport.dataflow_type = Push, Pull
   *  - dataport.subscription_type = Once, New, Periodic
   * 
   * OutPortRawTCPProvider declares the following:
   *  - dataport.interface_type = RawTCP
   *  - dataport.dataflow_type = Push, Pull
   *  - dataport.subscription_type = Once, New, Periodic
   * 
   * Finally, PortProfile::properties declares the following:
   *  - dataport.interface_type = CORBA_Any, RawTCP
   *  - dataport.dataflow_type = Push, Pull
   *  - dataport.subscription_type = Once, New, Periodic
   *
   * @since 0.4.0
   *
   * @endif
   */
  class OutPortProvider
    : public DataPortStatus
  {
  public:
    DATAPORTSTATUS_ENUM
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     * @endif
     */
    virtual ~OutPortProvider(void);

    /*!
     * @if jp
     * @brief ݒ菉
     *
     * InPortConsumer̊eݒsBNXł́A^ꂽ
     * PropertiesKvȏ擾ĊeݒsB init() 
     * ́AOutPortProviderエсAڑɂꂼĂ΂
     * \BāÅ֐͕Ă΂邱Ƃz肵ċL
     * qׂłB
     * 
     * @param prop ݒ
     *
     * @else
     *
     * @brief Initializing configuration
     *
     * This operation would be called to configure in initialization.
     * In the concrete class, configuration should be performed
     * getting appropriate information from the given Properties data.
     * This function might be called right after instantiation and
     * connection sequence respectivly.  Therefore, this function
     * should be implemented assuming multiple call.
     *
     * @param prop Configuration information
     *
     * @endif
     */
    virtual void init(coil::Properties& prop);

    /*!
     * @if jp
     * @brief obt@Zbg
     *
     * OutPortProviderf[^oobt@ZbgB
     * łɃZbgꂽobt@ꍇAȑÕobt@ւ
     * |C^ɑ΂ď㏑B
     * OutPortProvider̓obt@̏L肵ĂȂ̂ŁA
     * obt@̍폜̓[U̐ӔCōsȂ΂ȂȂB
     *
     * @param buffer OutPortProviderf[^oobt@ւ̃|C^
     *
     * @else
     * @brief Setting outside buffer's pointer
     *
     * A pointer to a buffer from which OutPortProvider retrieve data.
     * If already buffer is set, previous buffer's pointer will be
     * overwritten by the given pointer to a buffer.  Since
     * OutPortProvider does not assume ownership of the buffer
     * pointer, destructor of the buffer should be done by user.
     * 
     * @param buffer A pointer to a data buffer to be used by OutPortProvider
     *
     * @endif
     */
    virtual void setBuffer(CdrBufferBase* buffer) = 0;

    /*!
     * @if jp
     * @brief Xiݒ肷B
     * @else
     * @brief Set the listener. 
     * @endif
     */
    virtual void setListener(ConnectorInfo& info,
                             ConnectorListeners* listeners) = 0;

    /*!
     * @if jp
     * @brief Connectorݒ肷B
     * @else
     * @brief set Connector
     * @endif
     */
    virtual void setConnector(OutPortConnector* connector) = 0;

    /*!
     * @if jp
     * @brief InterfaceProfileJ
     *
     * InterfaceProfileJB
     * Ŏw肷vpeB NameValue IuWFNg
     * dataport.interface_type l𒲂ׁAY|[gɐݒ肳Ă
     * C^[tF[X^Cvƈvꍇ̂ݏ擾B
     *
     * @param properties InterfaceProfile󂯎vpeB
     *
     * @else
     * @brief Publish InterfaceProfile information
     *
     * Publish interfaceProfile information.
     * Check the dataport.interface_type value of the NameValue object 
     * specified by an argument in property information and get information
     * only when the interface type of the specified port is matched.
     *
     * @param properties Properties to get InterfaceProfile information
     *
     * @endif
     */
    virtual void publishInterfaceProfile(SDOPackage::NVList& properties);
    
    /*!
     * @if jp
     * @brief InterfaceJ
     *
     * InterfaceJBŎw肷vpeB
     * NameValue IuWFNgdataport.interface_type l𒲂ׁAY|[
     * gɐݒ肳ĂȂNameValue ɏǉBłɓC
     * ^[tF[Xo^ς݂̏ꍇ͉sȂB
     *
     * @param properties Interface󂯎vpeB
     *
     * @else
     * @brief Publish interface information
     *
     * Publish interface information.  Check the
     * dataport.interface_type value of the NameValue object specified
     * by an argument in the property information, and add the
     * information to the NameValue if this port is not specified.
     * This does not do anything if the same interface is already
     * subscribed.
     *
     * @param properties Properties to receive interface information
     *
     * @endif
     */
    virtual bool publishInterface(SDOPackage::NVList& properties);
    
  protected:
    /*!
     * @if jp
     * @brief |[g^Cvݒ肷
     *
     * Ŏw肵|[g^Cvݒ肷B
     *
     * @param port_type ݒΏۃ|[g^Cv
     *
     * @else
     * @brief Set the port type
     *
     * Set the port type specified by the argument.
     *
     * @param port_type The target port type to set
     *
     * @endif
     */
    void setPortType(const char* port_type);
    
    /*!
     * @if jp
     * @brief f[^^Cvݒ肷
     *
     * Ŏw肵f[^^Cvݒ肷B
     *
     * @param data_type ݒΏۃf[^^Cv
     *
     * @else
     * @brief Set the data type
     *
     * Set the data type specified by the argument.
     *
     * @param data_type The target data type to set
     *
     * @endif
     */
    void setDataType(const char* data_type);
    
    /*!
     * @if jp
     * @brief C^[tF[X^Cvݒ肷
     *
     * Ŏw肵C^[tF[X^Cvݒ肷B
     *
     * @param interface_type ݒΏۃC^[tF[X^Cv
     *
     * @else
     * @brief Set the interface type
     *
     * Set theinterface type specified by the argument.
     *
     * @param interface_type The target interface type to set
     *
     * @endif
     */
    void setInterfaceType(const char* interface_type);
    
    /*!
     * @if jp
     * @brief f[^t[^Cvݒ肷
     *
     * Ŏw肵f[^t[^Cvݒ肷B
     *
     * @param dataflow_type ݒΏۃf[^t[^Cv
     *
     * @else
     * @brief Set the data flow type
     *
     * Set the data flow type specified by the argument.
     *
     * @param dataflow_type The target data flow type to set
     *
     * @endif
     */
    void setDataFlowType(const char* dataflow_type);
    
    /*!
     * @if jp
     * @brief TuXNvV^Cvݒ肷
     *
     * Ŏw肵TuXNvV^Cvݒ肷B
     *
     * @param subs_type ݒΏۃTuXNvV^Cv
     *
     * @else
     * @brief Set the subscription type
     *
     * Set the subscription type specified by the argument.
     *
     * @param subs_type The target subscription type to set
     *
     * @endif
     */
    void setSubscriptionType(const char* subs_type);
    
  protected:
    /*!
     * @if jp
     * @brief |[gvt@CێvpeB
     * @else
     * @brief Properties to hold the port profiles
     * @endif
     */
    SDOPackage::NVList m_properties;
    /*!
     * @if jp
     * @brief K[Xg[
     * @else
     * @brief Logger stream
     * @endif
     */
    mutable Logger rtclog;
    
  private:
    std::string m_portType;
    std::string m_dataType;
    std::string m_interfaceType;
    std::string m_dataflowType;
    std::string m_subscriptionType;


  public:
    // functors
    /*!
     * @if jp
     * @brief C^[tF[Xvt@CJ邽̃t@N^
     * @else
     * @brief Functor to publish interface profile
     * @endif
     */
    struct publishInterfaceProfileFunc
    {
      publishInterfaceProfileFunc(SDOPackage::NVList& prop) : m_prop(prop) {}
      void operator()(OutPortProvider* provider)
      {
	provider->publishInterfaceProfile(m_prop);
      }
      SDOPackage::NVList& m_prop;
    };

    /*!
     * @if jp
     * @brief C^[tF[Xvt@CJ邽̃t@N^
     * @else
     * @brief Functor to publish interface profile
     * @endif
     */
    struct publishInterfaceFunc
    {
      publishInterfaceFunc(SDOPackage::NVList& prop)
        : m_prop(prop), provider_(0) {}
      void operator()(OutPortProvider* provider)
      {
	if (provider->publishInterface(m_prop))
          {
            provider_ = provider;
          }
      }
      SDOPackage::NVList& m_prop;
      OutPortProvider* provider_;
    };
  };

  typedef ::coil::GlobalFactory<OutPortProvider> OutPortProviderFactory;

}; // namespace RTC
#endif // RTC_OUTPORTPROVIDER_H
