// -*- C++ -*-
/*!
 * @file PublisherBase.h
 * @brief Publisher base class
 * @date $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherBase.h 1780 2010-01-25 10:47:26Z fsi-katami $
 *
 */

#ifndef RTC_PUBLISHERBASE_H
#define RTC_PUBLISHERBASE_H

#include <coil/Properties.h>
#include <coil/Factory.h>
#include <rtm/RTC.h>
#include <rtm/CdrBufferBase.h>
#include <rtm/DataPortStatus.h>

namespace coil
{
  class Properties;
}
namespace RTC
{
  class InPortConsumer;
  class ConnectorListeners;
  class ConnectorInfo;

  /*!
   * @if jp
   *
   * @class PublisherBase
   *
   * @brief Publisher NX
   * 
   * f[^o^C~OǗđo쓮Publisher* ̊NXB
   * e Publisher ͂̃NXpďڍׂB
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class PublisherBase
   *
   * @brief Base class of Publisher.
   *
   * This is a base class of Publisher*. This class manages data send timing.
   * Variation of Publisher* which implements details of Publisher inherits
   * this PublisherBase class.
   *
   * @endif
   */
  class PublisherBase
    : public DataPortStatus
  {
  public:
    DATAPORTSTATUS_ENUM
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~PublisherBase(void){};

    /*!
     * @if jp
     * @brief ݒ菉
     *
     * InPortConsumer̊eݒsBNXł́A^ꂽ
     * PropertiesKvȏ擾ĊeݒsB init() 
     * ́AOutPortProviderエсAڑɂꂼĂ΂
     * \BāÅ֐͕Ă΂邱Ƃz肵ċL
     * qׂłB
     * 
     * @param prop ݒ
     *
     * @else
     *
     * @brief Initializing configuration
     *
     * This operation would be called to configure in initialization.
     * In the concrete class, configuration should be performed
     * getting appropriate information from the given Properties data.
     * This function might be called right after instantiation and
     * connection sequence respectivly.  Therefore, this function
     * should be implemented assuming multiple call.
     *
     * @param prop Configuration information
     *
     * @endif
     */
    virtual ReturnCode init(coil::Properties& prop) = 0;
    /*!
     * @if jp
     * @brief InPortRV[}̃Zbg
     * @param consumer InPortRV[}
     * @else
     * @brief Store InPort consumer
     * @param consumer InPort consumer
     * @endif
     */
    virtual ReturnCode setConsumer(InPortConsumer* consumer) = 0;
    /*!
     * @if jp
     * @brief obt@̃Zbg
     * @param buffer CDRobt@
     * @else
     * @brief Setting buffer pointer
     * @param buffer CDR buffer
     * @endif
     */
    virtual ReturnCode setBuffer(BufferBase<cdrMemoryStream>* buffer) = 0;
    /*!
     * @if jp
     * @brief Xiݒ肷B
     * @param info ConnectorInfo
     * @param listeners ConnectorListeners 
     * @else
     * @brief Set the listener. 
     * @param info ConnectorInfo
     * @param listeners ConnectorListeners 
     * @endif
     */
    virtual ReturnCode setListener(ConnectorInfo& info,
                                   ConnectorListeners* listeners) = 0;
    /*!
     * @if jp
     * @brief f[^
     * @param data ރf[^ 
     * @param sec ^CAEg
     * @param nsec ^CAEg
     * @else
     * @brief Write data 
     * @param data Data
     * @param sec Timeout period
     * @param nsec Timeout period
     * @endif
     */
    virtual ReturnCode write(const cdrMemoryStream& data,
                             unsigned long sec,
                             unsigned long usec) = 0;

    /*!
     * @if jp
     *
     * @brief ANeBumF
     * 
     * ANeBuĂ邩mFB
     *
     * @return ԊmF(ANeBu:trueAANeBu:false)
     *
     * @else
     *
     * @brief Confirm to activate
     * 
     * Confirm that has been activated.
     *
     * @return Result of state confirmation
     *         (Active state:true, Inactive state:false)
     *
     * @endif
     */
    virtual bool isActive() = 0;
    /*!
     * @if jp
     * @brief ANeBu
     *
     * @else
     *
     * @brief activation
     *
     * @endif
     */
    virtual ReturnCode activate() = 0;
    /*!
     * @if jp
     * @brief ANeBu
     *
     * @else
     *
     * @brief deactivation
     *
     * @endif
     */
    virtual ReturnCode deactivate() = 0;


    

    /*!
     * @if jp
     *
     * @brief Publisher jB
     *
     * Y Publisher jB
     * Y Publisher svɂȂꍇ PublisherFactory ĂяoB
     *
     * @else
     *
     * @brief Release the Publisher
     *
     * Release this Publisher.
     * When Publisher becomes unnecessary, this is invoked from
     * PublisherFactory. 
     *
     * @endif
     */
    virtual void release(){}
  };

  typedef coil::GlobalFactory<PublisherBase> PublisherFactory;

};
#endif // RTC_PUBLISHERBASE_H
